"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import pytest

from django_s3.resource import Resource


@pytest.fixture(scope='class')
def resource():
    return Resource('F0001-B0001_320x320.SVG')



"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

import pytest
from django.utils.translation import ugettext_lazy as _

from django_s3.exceptions import ResourceError, ResourceNameError
from django_s3.resource import Resource

url_pattern = re.compile(r'^(?P<folder_name>[A-Z]{1,2}\d+[\w-]+)_[\w-]+\.(?P<extension>\w{3})$')


@pytest.mark.usefixtures('django_settings', 'resource')
class TestSource:
    def test_resource_creation(self, settings, resource):
        assert isinstance(resource, Resource)

    def test_creation_wrong_names(self, settings, resource):
        wrong_names = [
            '03323_12331-230x234.jgp',  # Starting with a number.
            ' 03323_12331-230x234.jgp',  # Starting with a space.
            '_3323_12331-230x234.jgp',  # Starting with a special character.
            'FFFF033_2312331-230x234.jgp',  # Starting with mora than 2 letters.
            'FF0332312331-230x234.jgp',  # Starting with mora than 2 letters.
            'fF0332312331-230x234.jgp',  # Missing '_' with a small letter.
            'FF03323_123:31-230x234.jgp',  # Containing spacial character in the middle.
            'FF3323_123\31-230x234.jgp',  # Containing spacial character in the middle (2).
        ]
        for sample in wrong_names:
            with pytest.raises(ResourceNameError):
                res = Resource(sample)

    def test_get_url(self, settings, resource):
        # Create a resource for each resource type and test for the correct url.
        resource_name_template = "{}001-B0001_320x320.SVG"
        for code, category in settings.S3_CATEGORY_MAP.items():
            res = Resource(resource_name_template.format(code))
            folder_name = url_pattern.match(res.name).groupdict()['folder_name']
            assert res.url == \
                   settings.S3_AWS_BASE_URL + settings.S3_BUCKET_NAME + '/' + category + \
                   '/{}/'.format(folder_name) + res.name

    def test_set_url(self, settings, resource):
        with pytest.raises(ResourceError) as exceinfo:
            resource.url = 'some value'
            assert _("Malformed reource name") == str(exceinfo.value)

    def test_get_name(self, settings, resource):
        assert 'F0001-B0001_320x320.SVG' == resource.name

    def test_set_url(self, settings, resource):
        with pytest.raises(ResourceError) as exceinfo:
            resource.name = 'some value'
            assert _("This attribute is readonly, is set at creation time.") == str(exceinfo.value)

    def test_url_wrong_file_name(self, settings, resource):
        """
        The object should never be created using a wrong name, since in the initializer,
        we check for name correctness.
        """
        pass

    def test_get_category(self, settings, resource):
        samples = [
            ('B0001_DEFAULT.JPG', 'BACKGROUND'),
            ('F0001_WHITE_DEFAULT.JPG', 'VASE'),
            ('F2344-B0001_DEFAULT.JPG', 'VASE'),
            ('CF0001_WHITE_DEFAULT.JPG', 'CLIPPING-FLOWER'),
            ('WF0001_WHITE_DEFAULT.JPG', 'WRAPPING-FLOWER'),
            ('CR01-047_B0001_320x320.JPG', 'CATALOGUE-PRODUCT'),
            ('PP0001_00000_0000.jpg', 'PERSONALIZED-PRODUCT'),
            ('CM991992_0000_000.png', 'COMPOSITION')
        ]
        for file_name, expected_category in samples:
            res = Resource(file_name)
            assert res.category == expected_category, \
                _("Expected category {} for the resource with name {}, see the settings "
                  "configuration: S3_CATEGORY_MAP.".format(expected_category, file_name))

    def test_get_category_code(self, settings, resource):
        samples = [
            ('B0001_DEFAULT.JPG', 'B'),
            ('F0001_WHITE_DEFAULT.JPG', 'F'),
            ('F2344-B0001_DEFAULT.JPG', 'F'),
            ('CF0001_WHITE_DEFAULT.JPG', 'CF'),
            ('WF0001_WHITE_DEFAULT.JPG', 'WF'),
            ('CR01-047_B0001_320x320.JPG', 'CR'),
            ('PP0001_00000_0000.jpg', 'PP'),
            ('CM991992_0000_000.png', 'CM')
        ]
        for file_name, expected_code in samples:
            res = Resource(file_name)
            assert res.category_code == expected_code, \
                _("Expected category {} for the resource with name {}, see the settings "
                  "configuration: S3_CATEGORY_MAP.".format(expected_code, file_name))

    def test_set_catgegory_code(self, settings, resource):
        with pytest.raises(ResourceError) as exceinfo:
            resource.category_code = 'some value'
            assert _("This attribute is readonly, see S3_CATEGORY_MAP.") == str(exceinfo.value)

    def test_get_code(self, settings, resource):
        # F0001-B0001_320x320.SVG
        assert resource.code == 'F0001-B0001_320x320', \
            "The is not being calculated correctly from the name."

    def test_set_code(self, settings, resource):
        with pytest.raises(ResourceError) as exceinfo:
            resource.category = ""
            assert _("This attribute readonly, It is calculated using the resource name") == str(exceinfo.value)



"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import logging
import os

import boto
from boto.s3.key import Key
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from django_s3.resource import Resource
from django_s3.s3_settings import django_s3_settings


class Transport(object):
    """
    Upload and download resources.
    """

    logger = logging.getLogger('django-s3-transport')

    def __init__(self):
        """
        Initializes the connection to the service.       
        """
        # connect to the bucket
        self.__conn = boto.connect_s3(
            aws_access_key_id=settings.S3_AWS_ACCESS_KEY_ID,
            aws_secret_access_key=settings.S3_AWS_SECRET_ACCESS_KEY
        )
        self.__bucket = self.__conn.get_bucket(settings.S3_BUCKET_NAME)
        self.__files = self.__bucket.list()

        # Map the key of each file to the name.
        self.__key_map = {}
        for key in self.__files:
            self.__key_map.update({key.key: key})

    def upload(self, resource):
        """
        Upload a resource.
        
        :param resource: An instance of `django_s3.resource.Resource`
        """
        try:
            key_holder = Key(self.__bucket)
            key_holder.key = "{}/{}".format(settings.S3_CATEGORY_MAP[resource.category_code], resource.name)
            key_holder.set_contents_from_filename(os.path.join(django_s3_settings.S3_UPLOAD_DIR_PATH, resource.name))
            key_holder.make_public()
        except Exception as err:
            Transport.logger.error(_("Error uploading file: {}. Error: {}".format(resource.name, err)))
            # Right now we don't know what exceptions are expected here, we propagate the error
            # up. If we found some exception then we'll add the proper handler.
            raise
        else:
            self.__files = self.__bucket.list()

    def download(self, resource):
        """
        Download a resource.
        
        :param resource: An instance of `django_s3.resource.Resource`
        :return: The absolute filename of the downloaded file. 
        """

        filename = os.path.join(django_s3_settings.S3_LOCAL_PATH, resource.name)

        # If the file exists do not download again.
        if not os.path.exists(filename):
            Transport.logger.info(_('Downloading {} to {}.'.format(resource.name, filename)))
            try:
                key_holder = Key(self.__bucket)
                key_holder.key = "{}/{}".format(settings.S3_CATEGORY_MAP[resource.category_code], resource.name)
                key_holder.get_contents_to_filename(filename)
            except Exception as err:
                Transport.logger.error(_("Error downloading file: {}. Error: {}".format(resource.name, err)))
                # Right now we don't know what exceptions are expected here, we propagate the error
                # up. If we found some exception then we'll add the proper handler.
                raise
        else:
            Transport.logger.info(_('File already exists, skipping download: {}'.format(filename)))

        return filename

    def download_by_name(self, name):
        """
        
        :param name: Name for the resource to download. 
        :return: A 2-tuple object containing a resource and the final path of the resource.
        """

        resource = Resource(name)
        filename = self.download(resource)

        return resource, filename



"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import tempfile

import mock
import pytest

from django_s3.s3_settings import DjangoS3SettingError


class TestSettings:
    @mock.patch('django_s3.s3_settings.settings', spec_set={})
    def test_missing_settings_error(self, mock_settings):
        with pytest.raises(DjangoS3SettingError):
            django_s3 = __import__('django_s3')
            # Accessing some configuration variable
            dir(django_s3.s3_settings.django_s3_settings)

    @mock.patch('django_s3.s3_settings.settings', autospec=True)
    def test_wrong_local_path_settings_error(self, mock_settings):
        mock_settings.S3_LOCAL_PATH = tempfile.mktemp()

        with pytest.raises(DjangoS3SettingError):
            django_s3 = __import__('django_s3')
            # Accessing some configuration variable
            dir(django_s3.s3_settings.django_s3_settings)

    @mock.patch('django_s3.s3_settings.settings', autospec=True)
    def test_wrong_upload_dir_path_settings_error(self, mock_settings):
        # We need an existing file name here in order to hit
        # the checkpoint for S3_UPLOAD_DIR_PATH.
        mock_settings.S3_LOCAL_PATH = '/'
        mock_settings.S3_UPLOAD_DIR_PATH = tempfile.mktemp()

        with pytest.raises(DjangoS3SettingError):
            django_s3 = __import__('django_s3')
            # Accessing some configuration variable
            dir(django_s3.s3_settings.django_s3_settings)



"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import pytest

from django_s3.browser import Browser, FileTreeNode


@pytest.mark.usefixtures('paths')
class TestBrowser:
    def test_build_jason(self, paths):
        expected_structure = {
            'EXAMPLE': {
                'example.txt': {},
                'Example_1': {
                    'example_1.txt': {}
                }
            },
            'GRAPHIC-RESOURCES': {
                'BACKGROUNDS': {},
                'CLIPPING-FLOWERS': {},
                'CUSTOM-PRODUCTS': {},
                'CUSTOMIZABLE-PRODUCTS': {},
                'GIFTS': {},
                'LINE-PRODUCTS': {},
                'RESOURCE-COMPOSITIONS': {},
                'VASES': {},
                'VASES-WITH-BACKGROUNDS': {},
                'WRAPPING-FLOWERS': {}
            },
            'RAW-PICTURES': {
                'BACKGROUNDS': {},
                'GIFTS': {},
                'PRODUCTS': {
                    'CR01': {},
                    'CR02': {},
                    'CR03': {}
                },
                'VASES': {},
                'WRAPPING-FLOWERS': {}
            }
        }
        browser = Browser(paths)
        assert browser.walk() == expected_structure, \
            "The structure for the folders is not correct."

    def test_browser_get_structure_relative(self, paths):
        browser = Browser(paths)
        assert browser.walk('EXAMPLE') == {'example.txt': {}, 'Example_1': {'example_1.txt': {}}}, \
            "walk function with parameter returns the structure relative to the folder"

    def test_file_dir_type(self, paths):
        browser = Browser(paths)

        node = browser.walk('EXAMPLE')
        assert node.type == FileTreeNode.DIR
        assert node['example.txt'].type == FileTreeNode.FILE
        assert node['Example_1'].type == FileTreeNode.DIR

        node = browser.walk('RAW-PICTURES/PRODUCTS/CR01')
        assert node.type == FileTreeNode.DIR

    def test_file_dir_type(self, paths):
        browser = Browser(paths)

        node = browser.walk('EXAMPLE')
        assert node.name == 'EXAMPLE'
        assert node['example.txt'].name == 'example.txt'



import tempfile

SECRET_KEY = 'FALSE-KEY-000'

S3_BUCKET_NAME = 'bucket-name'
S3_AWS_ACCESS_KEY_ID = 'AWS_ACCESS_KEY_ID'
S3_AWS_SECRET_ACCESS_KEY = 'AWS_SECRET_ACCESS_KEY'
S3_LOCAL_PATH = tempfile.gettempdir()
S3_UPLOAD_DIR_PATH = tempfile.gettempdir()
S3_AWS_BASE_URL = 'http://s3.amazonaws.com/'
S3_CATEGORY_MAP = {
    'B': 'BACKGROUND',
    'F': 'VASE',
    'CF': 'CLIPPING-FLOWER',
    'WF': 'WRAPPING-FLOWER',
    'CR': 'CATALOGUE-PRODUCT',
    'PP': 'PERSONALIZED-PRODUCT',
    'CM': 'COMPOSITION'
}

INSTALLED_APPS = [
    'django_s3',
]

MIDDLEWARE_CLASSES = (
)



"""
Copyright (2017) Raydel Miranda 

This file is part of Dajngo-S3.

    Dajngo-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Dajngo-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dajngo-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

from django.utils.translation import ugettext_lazy as _


class ResourceError(Exception):
    def __init__(self, message):
        super(ResourceError, self).__init__(message)


class ResourceNameError(ResourceError):
    def __init__(self, message=_("Malformed reource name")):
        super(ResourceError, self).__init__(message)



"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import pytest

PATHS = [
    "GRAPHIC-RESOURCES/",
    "GRAPHIC-RESOURCES/BACKGROUNDS/",
    "GRAPHIC-RESOURCES/CLIPPING-FLOWERS/",
    "GRAPHIC-RESOURCES/CUSTOM-PRODUCTS/",
    "GRAPHIC-RESOURCES/CUSTOMIZABLE-PRODUCTS/",
    "GRAPHIC-RESOURCES/GIFTS/",
    "GRAPHIC-RESOURCES/LINE-PRODUCTS/",
    "GRAPHIC-RESOURCES/RESOURCE-COMPOSITIONS/",
    "GRAPHIC-RESOURCES/VASES-WITH-BACKGROUNDS/",
    "GRAPHIC-RESOURCES/VASES/",
    "GRAPHIC-RESOURCES/WRAPPING-FLOWERS/",
    "EXAMPLE/example.txt",
    "EXAMPLE/Example_1/example_1.txt",
    "RAW-PICTURES/",
    "RAW-PICTURES/BACKGROUNDS/",
    "RAW-PICTURES/GIFTS/",
    "RAW-PICTURES/PRODUCTS/",
    "RAW-PICTURES/PRODUCTS/CR01/",
    "RAW-PICTURES/PRODUCTS/CR02/",
    "RAW-PICTURES/PRODUCTS/CR03/",
    "RAW-PICTURES/VASES/",
    "RAW-PICTURES/WRAPPING-FLOWERS/",
]


@pytest.fixture(scope='class')
def paths():
    return PATHS



import logging
import os

from django.conf import LazyObject
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

logger = logging.getLogger('django-s3')


class DjangoS3SettingError(Exception):
    pass


class DjangoS3Settings(object):
    def __init__(self):

        self.S3_LOCAL_PATH = getattr(settings, 'S3_LOCAL_PATH', None)
        self.S3_UPLOAD_DIR_PATH = getattr(settings, 'S3_UPLOAD_DIR_PATH', None)
        self.S3_BUCKET_NAME = getattr(settings, 'S3_BUCKET_NAME', None)
        self.S3_AWS_ACCESS_KEY_ID = getattr(settings, 'S3_AWS_ACCESS_KEY_ID', None)
        self.S3_AWS_SECRET_ACCESS_KEY = getattr(settings, 'S3_AWS_SECRET_ACCESS_KEY', None)
        self.S3_AWS_BASE_URL = getattr(settings, 'S3_AWS_BASE_URL', None)
        self.S3_CATEGORY_MAP = getattr(settings, 'S3_CATEGORY_MAP', None)

        for setting_name in ['S3_BUCKET_NAME',
                             'S3_AWS_ACCESS_KEY_ID',
                             'S3_AWS_SECRET_ACCESS_KEY',
                             'S3_CATEGORY_MAP',
                             'S3_AWS_BASE_URL',
                             'S3_UPLOAD_DIR_PATH']:
            if getattr(self, setting_name) is None:
                raise DjangoS3SettingError(_("You must to set {} configuration variable".format(setting_name)))

        if not os.path.exists(self.S3_LOCAL_PATH):
            raise DjangoS3SettingError(_("Directory not found: {}".format(self.S3_LOCAL_PATH)))

        if not os.path.exists(self.S3_UPLOAD_DIR_PATH):
            raise DjangoS3SettingError(_("Directory not found: {}".format(self.S3_UPLOAD_DIR_PATH)))


class DjangoS3LazySettings(LazyObject):
    def _setup(self, name=None):
        self._wrapped = DjangoS3Settings()


django_s3_settings = DjangoS3LazySettings()



"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from django_s3.exceptions import ResourceError, ResourceNameError
from django_s3.s3_settings import django_s3_settings

url_pattern = re.compile(r'^(?P<folder_name>[A-Z]{1,2}\d+[\w-]+)_[\w-]+\.(?P<extension>\w{3,4})$')
cat_pattern = re.compile(r'^(?P<category_code>[A-Z]+)\d+.+$')
code_pattern = re.compile(r'^(?P<code>.+)\..+$')
size_pattern = re.compile('(?P<size>\d+x\d+)')


class Resource(object):
    """
    Represents a resource in a bucket
    """

    def __init__(self, name):
        self.__name = name
        self.__category = None
        self.__category_code = None
        self.__url = None
        self.__code = None
        self.__extension = None
        self.__size = None

        # This is a control for checking the correctness of the file name, according
        # the convention specified by the API.
        self.__compute_url()

    def __compute_url(self):
        match = url_pattern.match(self.__name)
        if match is not None:

            BASE_URL = django_s3_settings.S3_AWS_BASE_URL
            BASE_URL = BASE_URL[:-1] if BASE_URL.endswith('/') else BASE_URL

            folder = settings.S3_CATEGORY_MAP[self.category_code]

            return "{}/{}/{}/{}/{}".format(
                BASE_URL,
                django_s3_settings.S3_BUCKET_NAME,
                folder,
                match.groupdict()['folder_name'],
                self.__name
            )
        else:
            raise ResourceNameError()

    @property
    def name(self):
        return self.__name

    @name.getter
    def name(self):
        return self.__name

    @name.setter
    def name(self, value):
        raise ResourceError(_("This attribute is readonly, is set at creation time."))

    @property
    def category(self):
        return self.__category

    @category.getter
    def category(self):
        match = cat_pattern.match(self.__name)
        if match is not None:
            return django_s3_settings.S3_CATEGORY_MAP[match.groupdict()['category_code']]

    @category.setter
    def category(self, value):
        raise ResourceError(_("This attribute is readonly, see S3_CATEGORY_MAP."))

    @property
    def category_code(self):
        return self.__category_code

    @category_code.getter
    def category_code(self):
        match = cat_pattern.match(self.__name)
        if match is not None:
            return match.groupdict()['category_code']

    @category_code.setter
    def category_code(self, value):
        raise ResourceError(_("This attribute is readonly, see S3_CATEGORY_MAP."))

    @property
    def url(self):
        return self.__url

    @url.getter
    def url(self):
        if self.__url is None:
            self.__url = self.__compute_url()
        return self.__url

    @url.setter
    def url(self, value):
        raise ResourceError(_("This attribute readonly, It is calculated using the resource name"))

    @property
    def code(self):
        return self.__code

    @code.getter
    def code(self):
        if self.__code is None:
            match = code_pattern.match(self.__name)
            if match is not None:
                self.__code = match.groupdict()['code']
            else:
                raise ResourceNameError()
        return self.__code

    @code.setter
    def code(self, value):
        raise ResourceError(_("This attribute readonly, It is calculated using the resource name"))

    @property
    def extension(self):
        return self.__extension

    @extension.getter
    def extension(self):
        if self.__extension is None:
            match = url_pattern.match(self.__name)
            if match is not None:
                self.__extension = match.groupdict()['extension']
            else:
                raise ResourceNameError()
        return self.__extension

    @extension.setter
    def extension(self, value):
        raise ResourceError(_("This attribute readonly, It is calculated using the resource name"))

    @property
    def size(self):
        return self.__size

    @size.getter
    def size(self):
        if self.__size is None:
            match = url_pattern.match(self.__name)
            if match is not None:
                self.__size = match.groupdict()['size']
            else:
                raise ResourceNameError()
            # At this moment self.__size is a str object, (Ex. "100x100")
            # we need something like (100, 100)
            self.__size = [int(elem) for elem in self.__size.split('x')]
        return self.__size

    @size.setter
    def size(self, value):
        raise ResourceError(_("This attribute readonly, It is calculated using the resource name"))



"""
Copyright (2017) Raydel Miranda 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

from os import path

from setuptools import setup

here = path.abspath(path.dirname(__file__))

long_description = ''

try:
    import pypandoc

    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError):
    long_description = ''

setup(
    name='real-django-s3',

    version='1.2.4',
    description='A simple API for download and upload graphics resources to/from a Amazon S3 bucket',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/RaydelMiranda/django-s3',

    # Author details
    author='Raydel Miranda',
    author_email='raydel.miranda.gomez@gmail.com',

    license='GPL',
    data_files=[('', ['README.md'])],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='development web django amazon s3',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=[
        'django_s3'
    ],

    include_package_data=True,

    install_requires=['boto==2.38.0', 'pytest', 'mock'],
)



"""
Copyright (2017) Raydel Miranda Gomez 

This file is part of Django-S3.

    Django-S3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Django-S3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Django-S3.  If not, see <http://www.gnu.org/licenses/>.
"""

import boto
from django.conf import settings


class FileTreeNode(dict):
    DIR = 1
    FILE = 0

    def __init__(self, type_=DIR, name='', *args, **kwargs):
        super(FileTreeNode, self).__init__(*args, **kwargs)
        self.type = type_
        self.name = name


def get_file_list():
    conn = boto.connect_s3(
        aws_access_key_id=settings.S3_AWS_ACCESS_KEY_ID,
        aws_secret_access_key=settings.S3_AWS_SECRET_ACCESS_KEY
    )
    bucket = conn.get_bucket(settings.S3_BUCKET_NAME)
    return [path.name for path in list(bucket.list())]


class Browser(object):
    def __init__(self, file_list=None):
        self.__structure = FileTreeNode()
        # Connect to the bucket to obtain the file list if is None
        if file_list is None:
            self.__files = get_file_list()
        else:
            self.__files = file_list

        self.__build_structure(self.__files)

    def __build_structure(self, files):

        paths = list(files)
        paths.sort(key=lambda x: len(x.split('/')))

        for path in paths:
            self.__arrange_path_within_structure(path.split('/'), self.__structure)

    def __arrange_path_within_structure(self, path, structure):
        if path == [] or path[0] == '':
            return
        if path[0] not in structure.keys():
            structure[path[0]] = FileTreeNode(type_=FileTreeNode.DIR, name=path[0])
        # Rectify type if is a file.
        if path[-1] != '' and path[-1] == path[0]:
            structure[path[0]].type = FileTreeNode.FILE
        self.__arrange_path_within_structure(path[1:], structure[path[0]])

    def walk(self, relative_to="/"):
        if relative_to == '/':
            return self.__structure
        else:
            parts = relative_to.split('/')
            parts = filter(lambda x: x != '', parts)
            node = self.__structure
            for p in parts:
                node = node[p]
            return node
